/*******************************************************************************

    TINY VOLT-AMMMETER

    VOLTMETRO  E AMPERMETRO

    C/ PIC12F675 E LCD (MTODO LPLEX - 3 FIOS)


    AUTOR: CLAUDIO LARIOS

    DATA:25/04/2016


    MEDE AT 10 AMPERES COM SHUNT DE 0,47 OHMS - 50 W
    MEDE AT 50 VOLTS DE ACORDO COM OPAO DO USURIO

    USO: FONTES DE ALIMENTAO DE BANCADA E OUTROS

    IMPORTANTE: USE CAPACITOR DE 220nF JUNTO AOS PINOS 1 E 8
    POIS O CIRCUITO  SENSVEL A INTERFERENCIAS PELA ALIMENTAO

    USO DIDTICO
    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG LARIOS.TECNOLOGIA.WS

*******************************************************************************/
#include <12F675.h>
#device adc=10
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT
#use fast_io(a)
//bytes
#byte  trisio = 0x85
#byte  gpio   = 0x05


//pinos do lcd (metodo Lariosplex - 3 fios)
#bit DB76  = 0x5.5  //pino 2 db7=direto db6=resistor 10k +cap 10nF ao gnd
#bit DB54  = 0x5.4  //pino 3 db5=direto db4=resistor 10k +cap 10nF ao gnd
#bit E_RS  = 0x5.1  //pino 6 E=direto RS=resistor 10k +cap 10nF ao gnd
#bit cal   = 0x5.3  //pino 4 boto de calibrao

#define vcarga 500 // por tentativa e erro (entre 200 a 500)
#define vdescarga 500//idem
#define v_delay_clk 2  // de 2 a 50 conforme LCD

#define RSHUNT 470 //miliohms (shunt de 0,47 ohms- 50W p/ medir 10 AMP )


#rom 0x2100 = { RSHUNT,RSHUNT>>8}//grava valor do 'shunt' na eeprom


//==============================================================================
//
//    Opo do usurio: Escolha o valor mximo mostrado na escala de tenso
//
//==============================================================================

 // #define v5max    //escala 0 - 5 volts (direto)

 // #define v10max  //escala 0 - 10 volts (somente pot10k)

 // #define v15max    //escala 0 - 15 volts (entrada com divisor resistivo 5k+pot10k)

 // #define v20max  //escala 0 - 20 volts (entrada com divisor resistivo 10k+pot10k)

 // #define v25max    //escala 0 - 25 volts (entrada com divisor resistivo 15k+pot10k)

    #define v30max  //escala 0 - 30 volts (entrada com divisor resistivo 20k+pot10k)

 // #define v35max    //escala 0 - 35 volts (entrada com divisor resistivo 25k+pot10k)

 // #define v40max  //escala 0 - 40 volts (entrada com divisor resistivo 30k+pot10k)

 // #define v45max  //escala 0 - 45 volts (entrada com divisor resistivo 35k+pot10k)

 // #define v50max  //escala 0 - 50 volts (entrada com divisor resistivo 40k+pot10k)


//Ateno: Descomente apenas uma nica opo acima e comente todas as restantes.
//(altere o resistor de entrada conforme a escala selecionada)
/*
   Formula Simplificada=  Rin= (Vin_max -10v)
   onde:
   Vin=  a tenso de final de escala desejada que ser mostrada no LCD
   Rin= valor do resistor (em Kohms) para obter o final da escala desejado

   (desde que o potenciometro usado seja de 10K, resultando em  ajuste com curso
   prximo do centro)



*******************************************************************************/

//variveis globais
unsigned int1 RS,f_cal;
unsigned int8 dez,un,vun,vdez,a;
unsigned int16 lido,res,calibra;
unsigned int32 soma;

/*******************************************************************************
                          SUBROTINAS
*******************************************************************************/


void descarregar(){ DB76=0;DB54=0;E_RS=0; delay_us(vdescarga);}
void t_carga(){delay_us(vcarga);}
void clk(){ E_RS=1; delay_cycles(v_delay_clk); E_RS=0;}//gera clock par Lcd


/*******************************************************************************
                          SUBROTINAS de LCD
*******************************************************************************/


/*******************************************************************************
                         ENVIA NIBBLE
*******************************************************************************/
void env_nible(unsigned int8 nib){
    descarregar();//descarrega capacitores
    nib&=0x0f;//limita
    if(nib&0x01)DB54=1;//testa bit 0
    if(nib&0x04)DB76=1;//testa bit 2
    if(RS)E_RS=1;else E_RS=0;//comando ou dado
    t_carga();
    if(nib&0x02)DB54=1; else DB54=0;//testa bit 1
    if(nib&0x08)DB76=1; else DB76=0;//testa bit 3
    clk();//clock

}

/*******************************************************************************
                          ENVIA DADOS PARA O LCD
*******************************************************************************/

void env_lcd_d(unsigned int8 dado){

 RS=1;//indica que enviar 'dados'
 env_nible(dado>>4);//high nibble primeiro
 env_nible(dado);
 delay_ms(4);
}

/*******************************************************************************
                         ENVIA COMANDOS PARA LCD
*******************************************************************************/
void env_lcd_c(unsigned int8 comando){

 RS=0;//indica que enviar 'comandos'
 env_nible(comando>>4);//high nibble primeiro
 env_nible(comando);
 delay_ms(4);
}

/*******************************************************************************
                         INICIALIZAO DO LCD
*******************************************************************************/

void Lcd_Init(){
  unsigned int8 a;

   descarregar();//descarrega capacitores
   RS=0; //enviar comando
   delay_ms(40);//delay inicial
   for(a=0;a<3;a++) { env_nible(3);delay_ms(5);}//envia 3x 0x03
   env_nible(2);//envia 0x02
   delay_ms(5);//delay 5 ms
   env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
   env_lcd_c(0x06);// no shift
   env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
   env_lcd_c(0x01);//limpa tela
}

/*******************************************************************************
        FAZ LEITURA DE UM CANAL DO CONVERSOR ANALGICO/DIGITAL
*******************************************************************************/
void get_canal(unsigned int8 q){
     set_adc_channel(q); //ajusta para o pino 5 (AN2) ou  7 (AN0)
     delay_us(10);
     lido=READ_ADC();// l valor da tenso de entrada do pino 5
     for(a=0;a<49;a++){//soma as 50 leituras sucessivas
     lido+=READ_ADC();
     }
     lido/=50;//mdia das ultimas 50 leituras
     soma= 5000* (int32) lido/1023; // obtem a tenso de entrada c/ vref 5v

}

/*******************************************************************************
               MOSTRA NO LCD A LEITURA PROCESSADA
*******************************************************************************/
void mostra_res(){
     if(!dez)dez=240;//se for zero mostrar espao
     env_lcd_d(dez+'0');
     env_lcd_d(un+'0');
     env_lcd_d(',');
     env_lcd_d(vun+'0');
     env_lcd_d(vdez+'0');
     env_lcd_d(' ');
}

/*******************************************************************************
                 CONVERTE DE HEXA PARA DECIMAL
*******************************************************************************/
void converte_res(){
    dez=res/10000;
     res=res%10000;
     un=res/1000;
     res=res%1000;
     vun= res/100;
     res=res%100;
     vdez=res/10;
}
/*******************************************************************************
                 DELAY DE 100 MS
*******************************************************************************/
void delay_dbc(){
  delay_ms(100);
}
/*******************************************************************************
                 ROTINA DE ESCRITA NA EEPROM
*******************************************************************************/
void escreve_eeprom(unsigned int8 end,unsigned int8 val){
    write_eeprom(end,val);
}



/*******************************************************************************
                        ROTINA MAIN
*******************************************************************************/

void main() {

   setup_adc_ports(sAN2 | VSS_VDD);  //seleciona AN2 e 5v como referncia
   setup_adc(ADC_CLOCK_DIV_8 );// ajusta o clock do adc
   set_adc_channel(2); //ajusta para o pino 5 (AN2)
   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);//desliga timer 1
   setup_comparator(NC_NC_NC_NC);//desliga comparadores
   setup_vref(FALSE);//desliga vref
   trisio=0b001101;//ajusta entradas e sadas
   gpio=0;//zera tudo
   f_cal=0;
   descarregar();//descarga dos capacitores do Lariosplex
   Lcd_Init();//inicializa Lcd
   printf(env_lcd_d("Tiny VoltAmmeter"));//mostra o nome
   env_lcd_c(0xc0);//linha 2
   printf(env_lcd_d("Larios.Tecnol.Ws"));// o blog
   delay_ms(2000);
   env_lcd_c(1);//apaga tela
   calibra= read_eeprom(0)| (int16) read_eeprom(1)<<8;//le o valor do shunt na EEprom
/*******************************************************************************
           LOOP PRINCIPAL DE REPETIO
*******************************************************************************/


   for(;;){

/*******************************************************************************
                        ROTINA DE CALIBRAO
*******************************************************************************/
//testa jumper ou boto de calibrao
  if(!cal){
     f_cal=!f_cal;//inverte estado
     if(f_cal){env_lcd_c(1); env_lcd_c(0x81); printf(env_lcd_d("Calibrar Amp:"));}
     else{
       env_lcd_c(1); //apaga tela

     }
     delay_dbc();
     while(!cal) delay_dbc();//aguarda solta boto
     escreve_eeprom(0,calibra); escreve_eeprom(1,calibra>>8);//salva novo ajuste

 }

 else{
   if(f_cal){ //se est em modo de calibrao, ento...
     get_canal(0);// obtem resultado tambm em 'lido'
     if(lido>900) calibra--; //se boto 'up' pressionado, diminue valor apresentado
     if(lido<50)  calibra++; //se boto 'down' pressionado, aumenta valor apresentado
     delay_dbc(); //delay de 100ms
   }
  }//if(!cal)

/*******************************************************************************
              ROTINA DE OPERAO DO AMPERMETRO
*******************************************************************************/
     get_canal(2);// obtem resultado em 'soma'
     soma*=1000;//ajusta p/ evitar usar frao
     soma/=calibra;  //onde: I=Vshunt/Rshunt
     res=soma;//int32 para int16
     converte_res();// ajusta para decimal
     //envia resultado para lcd
     env_lcd_c(0xc3); // 3 posico da linha 2
     env_lcd_d('I'); // letra 'I'
     env_lcd_d('='); // simbolo '='
     env_lcd_d(' '); //envia espao
     mostra_res(); //mostra resultado no LCD
     env_lcd_d('A');// letra 'A'
/*******************************************************************************
              ROTINA DE OPERAO DO VOLTMETRO
*******************************************************************************/
    if(!f_cal){ //se no estiver em modo de calibrao de Amperes

      get_canal(0);// obtem resultado em 'soma'
      
 
     res=soma;
     #ifdef v10max
     res*=2; //  Vmax entrada/10v,  ento: 10/5=2
     #endif
     #ifdef v15max
     res*=3; //  Vmax entrada/15v,  ento: 15/5=3
     #endif
     #ifdef v20max
     res*=4; //  Vmax entrada/20v,  ento: 20/5=4
     #endif
     #ifdef v25max
     res*=5; //  Vmax entrada/10v,  ento: 25/5=5
     #endif
     #ifdef v30max
     res*=6; //  Vmax entrada/30v,  ento: 30/5=6
     #endif
     #ifdef v35max
     res*=7; //  Vmax entrada/35v,  ento: 35/5=7
     #endif
     #ifdef v40max
     res*=8; //  Vmax entrada/40v,  ento: 40/5=8
     #endif
     #ifdef v45max
     res*=9; //  Vmax entrada/45v,  ento: 45/5=9
     #endif
     #ifdef v50max
     res*=10; //  Vmax entrada/50v,  ento: 50/5=10
     #endif
    
     converte_res();// ajusta para decimal
     //envia resultado para lcd
     env_lcd_c(0x83); // 3 posico da linha 1
     env_lcd_d('E'); // letra 'I'
     env_lcd_d('='); // simbolo '='
     env_lcd_d(' '); //envia espao
     mostra_res(); //mostra resultado no LCD
     env_lcd_d('V');// letra 'A'
    }

    delay_ms(100);// limita o nmero de leituras por segundo
  
   }//for(;;)

}//void main()
